@@ -4,12 +4,15 @@ from fsutil.metadata import (
     __author__, __copyright__, __description__,
     __email__, __license__, __title__, __version__, )
 
+import datetime as dt
 import errno
 import glob
 import hashlib
 import os
+import requests
 import shutil
 import sys
+import zipfile
 
 try:
     # python 2
@@ -200,6 +203,33 @@ def create_file(path, content='', overwrite=False):
     write_file(path, content)
 
 
+def create_zip_file(path, content_paths, overwrite=True, compression=zipfile.ZIP_DEFLATED):
+    """
+    Create zip file at path compressing directories/files listed in content_paths.
+    If overwrite is allowed and dest zip already exists, it will be overwritten.
+    """
+    assert_not_dir(path)
+    if not overwrite:
+        assert_not_exists(path)
+    make_dirs_for_file(path)
+
+    def _write_content_to_zip_file(file, path, basedir=''):
+        assert_exists(path)
+        if is_file(path):
+            filename = get_filename(path)
+            filepath = join_path(basedir, filename)
+            file.write(path, filepath)
+        elif is_dir(path):
+            for item_name in os.listdir(path):
+                item_path = join_path(path, item_name)
+                if is_dir(item_path):
+                    basedir = join_path(basedir, item_name)
+                _write_content_to_zip_file(file, item_path, basedir)
+
+    with zipfile.ZipFile(path, 'w', compression) as file:
+        for content_path in content_paths:
+            _write_content_to_zip_file(file, content_path)
+
 def delete_dir(path):
     """
     Alias for remove_dir.
@@ -208,6 +238,13 @@ def delete_dir(path):
     return removed
 
 
+def delete_dir_content(path):
+    """
+    Alias for remove_dir_content.
+    """
+    remove_dir_content(path)
+
+
 def delete_dirs(*paths):
     """
     Alias for delete_dirs.
@@ -230,6 +267,24 @@ def delete_files(*paths):
     remove_files(*paths)
 
 
+def download_file(url, dirpath, filename=None, chunk_size=8192):
+    """
+    Download a file from url to dirpath.
+    If filename is provided, the file will be named using filename.
+    """
+    # https://stackoverflow.com/a/16696317/2096218
+    filename = filename or get_filename(url) or 'download'
+    filepath = join_path(dirpath, filename)
+    make_dirs_for_file(filepath)
+    with requests.get(url, stream=True) as response:
+        response.raise_for_status()
+        with open(filepath, 'wb') as file:
+            for chunk in response.iter_content(chunk_size=chunk_size):
+                if chunk:
+                    file.write(chunk)
+    return filepath
+
+
 def exists(path):
     """
     Check if a directory of a file exists at the given path.
@@ -237,6 +292,20 @@ def exists(path):
     return os.path.exists(path)
 
 
+def extract_zip_file(path, dest, autodelete=False, content_paths=None):
+    """
+    Extract zip file at path to dest path.
+    If autodelete, the archive will be deleted after extraction.
+    If content_paths list is defined, only listed items will be extracted, otherwise all.
+    """
+    assert_file(path)
+    make_dirs(dest)
+    with zipfile.ZipFile(path, 'r') as file:
+        file.extractall(dest, members=content_paths)
+    if autodelete:
+        remove_file(path)
+
+
 def _filter_paths(basepath, relpaths, predicate=None):
     """
     Filter paths relative to basepath according to the optional predicate function.
@@ -246,11 +315,56 @@ def _filter_paths(basepath, relpaths, predicate=None):
     for relpath in relpaths:
         abspath = os.path.join(basepath, relpath)
         if predicate is None or predicate(abspath):
-            paths.append((relpath, abspath, ))
-    paths.sort(key=lambda path: path[0])
+            paths.append(abspath)
+    paths.sort()
     return paths
 
 
+def get_dir_creation_date(path):
+    """
+    Get the directory creation date.
+    """
+    assert_dir(path)
+    creation_timestamp = os.path.getctime(path)
+    creation_date = dt.datetime.fromtimestamp(creation_timestamp)
+    return creation_date
+
+
+def get_dir_creation_date_formatted(path, format='%Y-%m-%d %H:%M:%S'):
+    """
+    Get the directory creation date formatted using the given format.
+    """
+    date = get_dir_creation_date(path)
+    return date.strftime(format)
+
+
+def get_dir_last_modified_date(path):
+    """
+    Get the directory last modification date.
+    """
+    assert_dir(path)
+    last_modified_timestamp = os.path.getmtime(path)
+    for basepath, dirnames, filenames in os.walk(path):
+        for dirname in dirnames:
+            dirpath = os.path.join(basepath, dirname)
+            last_modified_timestamp = max(
+                last_modified_timestamp, os.path.getmtime(dirpath))
+        for filename in filenames:
+            filepath = os.path.join(basepath, filename)
+            last_modified_timestamp = max(
+                last_modified_timestamp, os.path.getmtime(filepath))
+    last_modified_date = dt.datetime.fromtimestamp(last_modified_timestamp)
+    return last_modified_date
+
+
+def get_dir_last_modified_date_formatted(path, format='%Y-%m-%d %H:%M:%S'):
+    """
+    Get the directory last modification date formatted using the given format.
+    """
+    date = get_dir_last_modified_date(path)
+    return date.strftime(format)
+
+
 def get_dir_size(path):
     """
     Get the directory size in bytes.
@@ -282,6 +396,24 @@ def get_file_basename(path):
     return basename
 
 
+def get_file_creation_date(path):
+    """
+    Get the file creation date.
+    """
+    assert_file(path)
+    creation_timestamp = os.path.getctime(path)
+    creation_date = dt.datetime.fromtimestamp(creation_timestamp)
+    return creation_date
+
+
+def get_file_creation_date_formatted(path, format='%Y-%m-%d %H:%M:%S'):
+    """
+    Get the file creation date formatted using the given format.
+    """
+    date = get_file_creation_date(path)
+    return date.strftime(format)
+
+
 def get_file_extension(path):
     """
     Get the file extension from the given path/url.
@@ -304,6 +436,24 @@ def get_file_hash(path, func='md5'):
     return hash_hex
 
 
+def get_file_last_modified_date(path):
+    """
+    Get the file last modification date.
+    """
+    assert_file(path)
+    last_modified_timestamp = os.path.getmtime(path)
+    last_modified_date = dt.datetime.fromtimestamp(last_modified_timestamp)
+    return last_modified_date
+
+
+def get_file_last_modified_date_formatted(path, format='%Y-%m-%d %H:%M:%S'):
+    """
+    Get the file last modification date formatted using the given format.
+    """
+    date = get_file_last_modified_date(path)
+    return date.strftime(format)
+
+
 def get_file_size(path):
     """
     Get the directory size in bytes.
@@ -495,6 +645,29 @@ def read_file(path, encoding='utf-8'):
     return content
 
 
+def read_file_from_url(url, **kwargs):
+    """
+    Read the content of the file at the given url.
+    """
+    response = requests.get(url, **kwargs)
+    response.raise_for_status()
+    content = response.text
+    return content
+
+
+def read_file_lines(path, strip_white=True, skip_empty=True, encoding='utf-8'):
+    """
+    Read file content lines according to the given options.
+    """
+    content = read_file(path, encoding)
+    lines = content.splitlines()
+    if strip_white:
+        lines = [line.strip() for line in lines]
+    if skip_empty:
+        lines = [line for line in lines if line]
+    return lines
+
+
 def remove_dir(path, **kwargs):
     """
     Remove a directory at the given path and all its content.
@@ -509,6 +682,15 @@ def remove_dir(path, **kwargs):
     return not exists(path)
 
 
+def remove_dir_content(path):
+    """
+    Removes all directory content (both sub-directories and files).
+    """
+    assert_dir(path)
+    remove_dirs(*list_dirs(path))
+    remove_files(*list_files(path))
+
+
 def remove_dirs(*paths):
     """
     Remove multiple directories at the given paths and all their content.
@@ -619,13 +801,23 @@ def split_filename(path):
 
 def split_filepath(path):
     """
-    Split a filename and returns its directory-path and filename.
+    Split a filepath and returns its directory-path and filename.
     """
     dirpath = os.path.dirname(path)
     filename = get_filename(path)
     return (dirpath, filename, )
 
 
+def split_path(path):
+    """
+    Split a path and returns its path-names.
+    """
+    head, tail = os.path.split(path)
+    names = head.split(os.sep) + [tail]
+    names = list(filter(None, names))
+    return names
+
+
 def write_file(path, content, append=False, encoding='utf-8'):
     """
     Write file with the specified content at the given path.
